pearson.test <- function(cross){
  sumrow <- rowSums(cross)
  sumcol <- colSums(cross)
  E <- outer(sumrow, sumcol, "*")/sum(cross)
  STATISTIC <- sum((cross - E)^2/E)
  PARAMETER <- (length(sumrow)-1)*(length(sumcol)-1)
  PVAL <- pchisq(STATISTIC, PARAMETER, lower = F)
  METHOD <- "Pearson's Chi-squared test"
  DNAME <- deparse(substitute(cross))
  names(STATISTIC) <- "Chi-squared"
  names(PARAMETER) <- "df"
  structure(list(statistic = STATISTIC, parameter = PARAMETER, 
    p.value = PVAL, method = METHOD, data.name = DNAME, observed = cross, 
    expected = E, residuals = (cross - E)/sqrt(E)), class = "htest")

###############################################################################
######                   Pearson's Chi-squared Test                       #####
###############################################################################
#
#   Argument
#     cross : a matrix(contingency table).
#
#   Value
#     statistic : the value the cumulative chi-squared test statistic.
#     parameter : the degrees of freedom of the approximate chi-squared
#                 distribution of the test statistic.
#     p.value   : the p-value for the test.
#     data.name : a character string giving the name(s) of the data.
#     observed  : the observed counts.
#     expected  : the expected counts under the null hypothesis.
#     residuals : the Pearson residuals,
#                 '(observed - expected) / sqrt(expected)'.
#
###############################################################################
#####                                                        by TAKANORI Tanase
#####                                                                  15/02/06
###############################################################################
}

cumchisq.test <- function(cross){
  STATISTIC <- lambda <- numeric(ncol(cross)-1)
  mat <- matrix(0, nrow(cross), 2)
  sumrow <- rowSums(cross)
  mat[,1] <- cross[,1]
  for(i in 1:nrow(cross))  mat[i,2] <- sum(cross[i,2:ncol(cross)])
  STATISTIC[1] <- as.vector(pearson.test(mat)$statistic)
  lambda[1] <- colSums(mat)[1]/colSums(mat)[2]
  for(i in 2:(ncol(cross)-1)){
    for(j in 1:nrow(cross)){
      mat[j,1] <- mat[j,1] + cross[j,i]
      mat[j,2] <- mat[j,2] - cross[j,i]
    }
    STATISTIC[i] <- as.vector(pearson.test(mat)$statistic)
    lambda[i] <- colSums(mat)[1]/colSums(mat)[2]
  }
  d <- 0
  for(i in 2:(ncol(cross)-1)) d <- d + lambda[1:(i-1)]/lambda[i]
  d <- 1+2*d/(ncol(cross)-1)
  nu <- (nrow(cross)-1)*(ncol(cross)-1)/d
  STATISTIC <- sum(STATISTIC)
  METHOD <- "Hirotsu's Cumulative Chi-squared Test"
  PARAMETER <- c(d, nu)
  names(STATISTIC) <- "Cumulative Chi-squared"
  names(PARAMETER) <- c("d", "nu")
  DNAME <- deparse(substitute(cross))
  PVAL <- 1-pchisq(sum(STATISTIC)/d, nu)
  structure(list(statistic = STATISTIC, parameter = PARAMETER, p.value = PVAL,
    method = METHOD, data.name = DNAME, observed = cross), class = "htest")

###############################################################################
######              Hirotsu's Cumulative Chi-squared Test                 #####
###############################################################################
#
#   Argument
#     cross : a matrix(contingency table).
#
#   Value
#     statistic : the value the cumulative chi-squared test statistic.
#     parameter : "d" is a fixed number.
#                 "nu" is  a degree of freedom of the approximate chi-squared 
#                  distribution of the test statistic.
#     p.value   : the p-value for the test.
#     data.name : a character string giving the name(s) of the data.
#     observed  : the observed counts.
#
###############################################################################
#####                                                        by TAKANORI Tanase
#####                                                                  15/02/06
###############################################################################
}

pearson.closed.test <- function(cross, alpha = 0.05){
# this needs package 'gtools' and function 'is.subset'.
  rows <- matrix(1:nrow(cross), ncol=nrow(cross), nrow=1)
  for(i in (nrow(cross)-1):2){
    num <- factorial(nrow(cross))/(factorial(nrow(cross)-i)*factorial(i))
    rows <- rbind(rows, matrix(0, ncol = nrow(cross), nrow = num))
    rows[(nrow(rows)-num+1):nrow(rows),(1:i)] <- combinations(nrow(cross), i)
  }
  cp <- numeric(nrow(cross))
  for(i in 1:length(cp)){
    if(i == nrow(cross) || i == nrow(cross)-1) cp[i] <- alpha
    else cp[i] <- 1-(1-alpha)^(i/nrow(cross))
  }
  res <- numeric(nrow(rows))
  if(pearson.test(cross)$p.value < alpha){
    res[1] <- 1
    i <- 2
    for(j in (nrow(cross)-1):2){
      num <- factorial(nrow(cross))/(factorial(nrow(cross)-j)*factorial(j))
      for(k in i:(i+num-1)){
        if(res[k] == 0){
          if(pearson.test(cross[rows[k,1:j],])$p.value < cp[j]) res[k] <- 1
          else if(j != 2){
            res[k] <- 2
            for(l in (i+num):nrow(rows)){
              if(is.subset(rows[l,], rows[k,]) == T) res[l] <- 3
            }          
          }
        }
      }
      i <- k+1
    } 
  }
  RESULT <- NULL
  num <- (length(res)-factorial(nrow(cross))/(factorial(nrow(cross)-2)*factorial(2))) + 1
  for(i in num:length(res)){
    if(res[i] == 1) RESULT <- rbind(RESULT, rows[i,1:2])
  }
  if(is.null(RESULT) == F){
    PVAL <- numeric(nrow(RESULT))
    for(i in 1:nrow(RESULT)) PVAL[i] <- pearson.test(cross[RESULT[i,],])$p.value
  } else {
    RESULT <- "No rows was rejected."
    PVAL <- NULL
  }
  list(result = RESULT, p.value = PVAL)
###############################################################################
######           Pearson Chi-squared Closed Testing Procedure             #####
###############################################################################
#
#   Argument
#     cross : a matrix(contingency table).
#     alpha : significance level(defalt; alpha = 0.05)
#
#   Value
#     result  : the rows rejected by Wishart matrix.
#               if result is NULL, no rows was rejedted.
#     p.value : the p-value for the test.
###############################################################################
#####                                                        by TAKANORI Tanase
#####                                                                  15/02/06
###############################################################################
}

hirotsu_scheffe.test <- function(cross, alpha = 0.05){
  if(alpha == 0.05){
    wishart <- matrix(c(
              8.5949 ,8.6365 ,8.7627 ,8.9763 ,9.2758 ,9.6484 ,10.0722,10.5272,11.0010,
              10.7403,10.8100,11.0196,11.3659,11.8320,12.3854,12.9915,13.6280,14.2834,
              12.6771,12.7766,13.0728,13.5521,14.1776,14.8975,15.6703,16.4732,17.2953,
              14.4883,14.6196,15.0052,15.6176,16.3968,17.2744,18.2045,19.1648,20.1251,
              16.2149,16.3780,16.8558,17.6013,18.5296,19.5585,20.6397,21.7514,22.8631,
              17.8781,18.0748,18.3925,19.5250,20.5988,21.7742,23.0021,24.2607,25.5193,
              19.4917,19.7234,20.3913,21.4017,22.6180,23.9364,25.3074,26.7094,28.1114,
              21.0697,21.3373,22.1028,23.2447,24.6010,26.0598,27.5715,29.1143,30.6571)
              , ncol=9, byrow=T)
  } else if(alpha == 0.01){
    wishart <- matrix(c(
              12.1602,12.2711,12.5950,13.1016,13.7392,14.4547,15.2125,15.9950,16.7934,
              14.5681,14.7283,15.1884,15.8857,16.7332,17.6608,18.6304,19.6250,20.6261,
              16.7268,16.9363,17.5293,18.4051,19.4433,20.5623,21.7235,22.9103,24.1139,
              18.7348,18.9943,19.7186,20.7648,21.9818,23.2799,24.6207,25.9875,27.3543,
              20.6391,20.9493,21.8040,23.0149,24.4023,25.8712,27.3833,28.9218,30.4603,
              24.2339,24.6479,25.7613,27.2889,29.0000,30.7935,32.6313,34.4963,36.3613,
              25.9326,26.4000,27.6426,29.3217,31.1867,33.1348,35.1276,37.1480,39.1684,
              27.6313,28.1521,29.5239,31.3544,33.3734,35.4760,37.6239,39.7997,41.9755)
	      , ncol=9, byrow=T)
  } else wishart <- NULL
  if(is.null(wishart) == F){
    x <- y <- Y <- numeric(3)
    N <- sum(cross)
    stat <- res <- matrix(0, nrow(cross), nrow(cross))
    wishart <- wishart[nrow(cross)-2,]
    for(j in 1:nrow(cross)){
      for(k in j:nrow(cross)){
        for(l in 1:(ncol(cross)-1)){
          stat[j,k] <- stat[j,k] +
          ((1/sum(cross[,1:l])+1/(N-sum(cross[,1:l])))*
          (sum(cross[j,1:l])/sum(cross[j,])-sum(cross[k,1:l])/sum(cross[k,]))^2)
        }
        stat[j,k] <- stat[k,j] <- stat[j,k]*N*(1/sum(cross[j,])+1/sum(cross[k,]))^(-1)
      }
    }
    element <- sqrt(sum(cross[,1])*sum(cross[,3]))/
    sqrt((sum(cross[,1])+sum(cross[,2]))*(sum(cross[,2])+sum(cross[,3])))
    if(element <= 0.15){
      x[1] <- 0.0
      x[2] <- 0.1
      x[3] <- 0.2
      y[1] <- wishart[1]
      y[2] <- wishart[2]
      y[3] <- wishart[3]
    } else if(0.15 < element && element <= 0.25){
      x[1] <- 0.1
      x[2] <- 0.2
      x[3] <- 0.3
      y[1] <- wishart[2]
      y[2] <- wishart[3]
      y[3] <- wishart[4]
    } else if(0.25 < element && element <= 0.35){
      x[1] <- 0.2
      x[2] <- 0.3
      x[3] <- 0.4
      y[1] <- wishart[3]
      y[2] <- wishart[4]
      y[3] <- wishart[5]
    } else if(0.35 < element && element <= 0.45){
      x[1] <- 0.3
      x[2] <- 0.4
      x[3] <- 0.5
      y[1] <- wishart[4]
      y[2] <- wishart[5]
      y[3] <- wishart[6]
    } else if(0.45 < element && element <= 0.55){
      x[1] <- 0.4
      x[2] <- 0.5
      x[3] <- 0.6
      y[1] <- wishart[5]
      y[2] <- wishart[6]
      y[3] <- wishart[7]
    } else {
      x[1] <- 0.5
      x[2] <- 0.6
      x[3] <- 0.7
      y[1] <- wishart[6]
      y[2] <- wishart[7]
      y[3] <- wishart[8]
    }
    for(j in 1:3){
      Y[j] <- 1
      for(k in 1:3){
        if(j != k) Y[j] <- Y[j]*(element-x[k]) / (x[j]-x[k])
      }
      Y[j] <- Y[j]*y[j]
    }
    cv <- sum(Y)
    for(j in 1:nrow(cross)){
      for(k in j:nrow(cross)){
        if(stat[j,k] >= cv) res[j,k] = res[k,j] = 1
        else res[j,k] = res[k,j] = 0
      }
    }
    if(sum(res) != 0){
      RESULT <- matrix(0, sum(res)/2, 2)
      i <- 1
      for(j in 1:nrow(cross)){
        for(k in j:nrow(cross)){
          if(res[j,k] == 1){
            RESULT[i,1] <- j
            RESULT[i,2] <- k
            i <- i + 1
          }
        }
      }
    }
    else RESULT <- NULL
  } else {
    RESULT <- "Alpha must equal to 0.05 or 0.01."
  }
  list(statistics = stat, result = RESULT)
###############################################################################
######      Hirotsu's Multiple Comparison Procedure (Scheffe Method)     ######
###############################################################################
#   
#   the number of row is from 3 to 10.
#   the number of col is only 3.
#
#   Argument
#     cross : a matrix(contingency table).
#     alpha : significance level, 0.05 or 0.01(defalt; alpha = 0.05).
#
#   Value
#     statistic : the value the cumulative chi-squared test statistic.
#     result : the rows rejected by Wishart matrix.
#              if result is NULL, no rows was rejedted.
###############################################################################
#####                                                        by TAKANORI Tanase
#####                                                                  15/02/06
###############################################################################
}

hirotsu_tukey.test <- function(cross){
  ul <- function(stat, sumrow, d, nu){
    p2 <- 0
    p1 <- 0
    rho <- sqrt(sumrow[2]*sumrow[3]/((sumrow[1]+sumrow[2])*(sumrow[1]+sumrow[3])))
    for(j in 0:10) p1 <- p1 + gamma(nu/2+j)*rho^(2*j)*(1-pchisq((stat/d)/(1-rho^2),nu+2*j))^2/factorial(j)
    p2 <- p2 + p1 * (1-rho^2)^(nu/2)/gamma(nu/2)
    p1 <- 0
    rho <- sqrt(sumrow[1]*sumrow[3]/((sumrow[1]+sumrow[2])*(sumrow[2]+sumrow[3])))
    for(j in 0:10) p1 <- p1 + gamma(nu/2+j)*rho^(2*j)*(1-pchisq((stat/d)/(1-rho^2),nu+2*j))^2/factorial(j)
    p2 <- p2 + p1 * (1-rho^2)^(nu/2)/gamma(nu/2)
    p1 <- 0
    rho <- sqrt(sumrow[1]*sumrow[2]/((sumrow[1]+sumrow[3])*(sumrow[2]+sumrow[3])))
    for(j in 0:10) p1 <- p1 + gamma(nu/2+j)*rho^(2*j)*(1-pchisq((stat/d)/(1-rho^2),nu+2*j))^2/factorial(j)
    p2 <- p2 + p1 * (1-rho^2)^(nu/2)/gamma(nu/2)
    3*(1-pchisq(stat/d, nu)) - p2
  }
  stat <- matrix(0, nrow(cross[1:3,]), nrow(cross[1:3,]))
  N <- sum(cross)
  for(i in 1:3){
    for(j in i:3){
      for(k in 1:(ncol(cross)-1)){
        stat[i,j] <- stat[i,j] + ((1/sum(cross[,1:k])+1/(N-sum(cross[,1:k])))*(sum(cross[i,1:k])/sum(cross[i,])-sum(cross[j,1:k])/sum(cross[j,]))^2)
      }
      stat[i,j] <- stat[j,i] <- stat[i,j]*N*(1/sum(cross[i,])+1/sum(cross[j,]))^(-1)
    }
  }
  sumrow <- rowSums(cross)
  d <- 1 + (sum(cross[,1])/sum(cross[,2:3])) / (sum(cross[,1:2])/sum(cross[,3]))
  nu <- 2 / d
  ll <- ul(stat, sumrow, d, nu)
  ll[1,1] <- ll[2,2] <- ll[3,3] <- 0
  list(statistics = stat, lower.limit = ll)
###############################################################################
######       Hirotsu's Multiple Comparison Procedure (Tukey Method)       #####
###############################################################################
#
#   the number of row is only 3.
#
#   Argument
#     cross : a matrix(contingency table).
#
#   Value
#     lower.limit : the approximated lower limits of significance propability.
#
###############################################################################
#####                                                        by TAKANORI Tanase
#####                                                                  15/02/06
###############################################################################
}

cumchisq.closed.test <- function(cross, alpha = 0.05){
# this needs package 'gtools' and function 'is.subset'.
  rows <- matrix(1:nrow(cross), ncol=nrow(cross), nrow=1)
  for(i in (nrow(cross)-1):2){
    num <- factorial(nrow(cross))/(factorial(nrow(cross)-i)*factorial(i))
    rows <- rbind(rows, matrix(0, ncol = nrow(cross), nrow = num))
    rows[(nrow(rows)-num+1):nrow(rows),(1:i)] <- combinations(nrow(cross), i)
  }
  cp <- numeric(nrow(cross))
  for(i in 1:length(cp)){
    if(i == nrow(cross) || i == nrow(cross)-1) cp[i] <- alpha
    else cp[i] <- 1-(1-alpha)^(i/nrow(cross))
  }
  res <- numeric(nrow(rows))
  if(cumchisq.test(cross)$p.value < alpha){
    res[1] <- 1
    i <- 2
    for(j in (nrow(cross)-1):2){
      num <- factorial(nrow(cross))/(factorial(nrow(cross)-j)*factorial(j))
      for(k in i:(i+num-1)){
        if(res[k] == 0){
          if(cumchisq.test(cross[rows[k,1:j],])$p.value < cp[j]) res[k] <- 1
          else if(j != 2){
            res[k] <- 2
            for(l in (i+num):nrow(rows)){
              if(is.subset(rows[l,], rows[k,]) == T) res[l] <- 3
            }          
          }
        }
      }
      i <- k+1
    } 
  }
  RESULT <- NULL
  num <- (length(res)-factorial(nrow(cross))/(factorial(nrow(cross)-2)*factorial(2))) + 1
  for(i in num:length(res)){
    if(res[i] == 1) RESULT <- rbind(RESULT, rows[i,1:2])
  }
  if(is.null(RESULT) == F){
    PVAL <- numeric(nrow(RESULT))
    for(i in 1:nrow(RESULT)) PVAL[i] <- cumchisq.test(cross[RESULT[i,],])$p.value
  } else {
    RESULT <- "No rows was rejected."
    PVAL <- NULL
  }
  list(result = RESULT, p.value = PVAL)
###############################################################################
######          Cumulative Chi-squared Closed Testing Procedure           #####
###############################################################################
#
#   Argument
#     cross : a matrix(contingency table).
#     alpha : significance level(defalt; alpha = 0.05)
#
#   Value
#     result  : the rows rejected by Wishart matrix.
#               if result is NULL, no rows was rejedted.
#     p.value : the p-value for the test.
###############################################################################
#####                                                        by TAKANORI Tanase
#####                                                                  15/02/06
###############################################################################
}

is.subset <- function(x, y){
# this is needed by 'pearson.closed.test' and 'cumchisq.closed.test'.
#                                         by TAKANORI Tanase 15/02/06
  i <- 1
  while(x[i] != 0) i <- i+1
  x <- x[1:(i-1)]
  i <- 1
  while(y[i] != 0) i <- i+1
  y <- y[1:(i-1)]
  res <- numeric(length(x))
  for(i in 1:length(x)){
    for(j in 1:length(y)){
      if(x[i] == y[j]) res[i] <- 1
    }
  }
  if(prod(res) == 1) return <- TRUE
  else return <- FALSE
  return
}
