IRT.trait <- function(a, b, u, method = "Newton") {
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<
#>>>>                                              <<<<<
#>>>>   Estimate characteristic parameter in IRT   <<<<<
#>>>>       ; Maximum likelihood estimate method   <<<<<
#>>>>                                              <<<<<
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<
   THETA <- SE <- NULL
   if(length(a) != length(b) || length(b) != ncol(u) || ncol(u) != length(a)) {
      stop("unequal number of lengthes in IRT")
   }
   for(i in 1:nrow(u)) {
      if(sum(u[i,]) == 0) {
         for(j in 1:length(b)) {
            if(b[j] == min(b)) {
               u[i,j] <- 0.5
            }
         }
      } else if(sum(u[i,]) == length(b)) {
         for(j in 1:length(b)) {
            if(b[j] == max(b)) {
               u[i,j] <- 0.5
            }
         }
      }
   }
   for(i in 1:nrow(u)) {
      if(method == "Newton") {
         theta <- 0
         while(length(theta) <= 2 || abs(rev(theta)[2] - rev(theta)[1]) > 0.00001) {
            theta <- append(theta,rev(theta)[1] - g.func(a, b, rev(theta)[1], u[i,]) / h.func(a, b, rev(theta)[1], u[i,]))
            Theta <- rev(theta)[1]
         }
      } else if(method == "Bisection") { 
         theta <- c(-3,3)
         while(g.func(a, b, theta[1], u[i,]) < 0) {
            theta[1] <- theta[1] - 1
         }
         while(g.func(a, b, theta[2], u[i,]) > 0) {   
            theta[2] <- theta[2] + 1
         }
         Theta <- theta[1]
         while(abs(g.func(a, b, Theta, u[i,])) > 0.00001) {
            Theta <- (theta[1] + theta[2]) / 2
            if(g.func(a, b, Theta, u[i,]) > 0) {
               theta[1] <- Theta
            } else {
               theta[2] <- Theta
            }
         }
      }
      THETA <- append(THETA, Theta)
      SE <- append(SE, 1 / sqrt(1.7 ^ 2 * sum(a ^ 2 * charfunc(a, b, Theta) * (1 - charfunc(a, b, Theta)))))
   }
   list(THETA = THETA, SE = SE)
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#
#  >>> input <<<
#           a: discrimination parameter vector
#           b: difficulty parameter vector
#           u: item response data ; matrix
#      method: A criterion among the following list;
#             Newton: Newton method
#          Bisection: Bisection method
#
#  >>> output <<<
#       THETA: estimate characteristic parameter
#          SE: standard error of estimate characteristic 
#  
#
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#                                               02/25/04  by  Tanase
}

IRT.bayes <- function(a, b, u, mu = 0, sigma = 1) {
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<
#>>>>                                              <<<<<
#>>>>   Estimate characteristic parameter in IRT   <<<<<
#>>>>                    ; Bayes estimate method   <<<<<
#>>>>                                              <<<<<
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<
   EAP <- SE <- NULL
   Aq <- c(-3.2, -2.8, -2.4, -2.0, -1.6, -1.2, -0.8, -0.4, 0, 0.4, 0.8, 1.2, 1.6, 2.0, 2.4, 2.8, 3.2)
   if(length(a) != length(b) || length(b) != ncol(u) || ncol(u) != length(a)) {
      stop("unequal number of lengthes in IRT")
   }
   for(i in 1:nrow(u)) {
      bayes <- f.func(a, b, Aq, u[i,]) * dnorm(Aq, mu, sigma) / sum(f.func(a, b, Aq, u[i,]) * dnorm(Aq, mu, sigma))
      EAP <- append(EAP, sum(Aq * bayes)) 
      SE <- append(SE, sum((Aq - rev(EAP)[1]) ^ 2 * bayes))
   }
   list(EAP = EAP, SE = sqrt(SE))
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#
#  >>> input <<<
#           a: discrimination parameter vector
#           b: difficulty parameter vector
#           u: item response data ; matrix
#          mu: mean of popular distribution
#       sigma: standard deviation of popular distribution
#
#  >>> output <<<
#         EAP: EAP estimate
#          SE: standard error of EAP estimate
#  
#
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#                                               02/25/04  by  Tanase
}

IRT.dist <- function(a, b, u) {
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#>>>>                                                        <<<<<
#>>>>   Estimate parameter of popular distribution in IRT    <<<<<
#>>>>                                                        <<<<<
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
   Aq <- c(-3.2, -2.8, -2.4, -2.0, -1.6, -1.2, -0.8, -0.4, 0, 0.4, 0.8, 1.2, 1.6, 2.0, 2.4, 2.8, 3.2)
   mu <- 0
   sigma <- 1
   if(length(a) != length(b) || length(b) != ncol(u) || ncol(u) != length(a)) {
      stop("unequal number of lengthes in IRT")
   }
   hiq <- matrix(0, ncol = length(Aq), nrow = nrow(u))
   while(length(mu) <= 2|| abs(rev(mu)[2] - rev(mu)[1]) > 0.0001 || abs(rev(sigma)[2] - rev(sigma)[1]) > 0.0001) {
      hq <- c(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0) 
      for(i in 1:nrow(u)) {
         hiq[i,] <- mlfunc(a, b, Aq, u[i,], rev(mu)[1], rev(sigma)[1]) / sum(mulfunc(a, b, Aq, u[i,], rev(mu)[1], rev(sigma)[1]))
         hq <- hq + hiq[i,]
      }
      mu <- append(mu, sum(Aq * hq) / nrow(u))
      sigma <- append(sigma, sqrt(sum((Aq - rev(mu)[2]) ^ 2 * hq) / nrow(u)))
   }
   list(mu = rev(mu)[1], sigma = rev(sigma)[1])
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#
#  >>> input <<<
#           a: discrimination parameter vector
#           b: difficulty parameter vector
#           u: item response data ; matrix
#
#  >>> output <<<
#          mu: mean of popular distribution
#       sigma: standard deviation of popular distribution
#  
#
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#                                               02/25/04  by  Tanase
}

IRT.dist.freq <- function(a, b, u, k) {
#>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#>>>>                                                        <<<<<
#>>>>   Estimate parameter of popular distribution in IRT    <<<<<
#>>>>                                                        <<<<<
#>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
   Aq <- c(-3.2, -2.8, -2.4, -2.0, -1.6, -1.2, -0.8, -0.4, 0, 0.4, 0.8, 1.2, 1.6, 2.0, 2.4, 2.8, 3.2)
   mu <- 0
   sigma <- 1
   if(length(a) != length(b) || length(b) != ncol(u) || ncol(u) != length(a)) {
      stop("unequal number of lengthes in IRT")
   }
   hiq <- matrix(0, ncol = length(Aq), nrow = nrow(u))
   while(length(mu) <= 2 || abs(rev(mu)[2] - rev(mu)[1]) > 0.0001 || abs(rev(sigma)[2] - rev(sigma)[1]) > 0.0001) {
      hq <- c(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0) 
      for(i in 1:nrow(u)) {
         hiq[i,] <- mlfunc(a, b, Aq, u[i,], rev(mu)[1], rev(sigma)[1]) / sum(mlfunc(a, b, Aq, u[i,], rev(mu)[1], rev(sigma)[1]))
         hq <- hq + k[i] * hiq[i,]
      }
      mu <- append(mu, sum(Aq * hq) / sum(k))
      sigma <- append(sigma, sqrt(sum((Aq - rev(mu)[2]) ^ 2 * hq) / sum(k)))
   }
   list(mu = rev(mu)[1], sigma = rev(sigma)[1])
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#
#  >>> input <<<
#           a: discrimination parameter vector
#           b: difficulty parameter vector
#           u: item response data ; matrix
#           k: freqquency vector
#
#  >>> output <<<
#          mu: mean of popular distribution
#       sigma: standard deviation of popular distribution
#  
#
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#                                               02/25/04  by  Tanase
}

IRT.par <- function(u, mu, sigma) {
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<
#>>>>                                              <<<<<
#>>>>   Estimate item parameter in IRT             <<<<<
#>>>>                                              <<<<<
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<
   Aq <- c(-3.2, -2.8, -2.4, -2.0, -1.6, -1.2, -0.8, -0.4, 0, 0.4, 0.8, 1.2, 1.6, 2.0, 2.4, 2.8, 3.2)
   x <- matrix(1, ncol = ncol(u), nrow = 1)
   y <- a.SE <- b.SE <- matrix(0, ncol = ncol(u), nrow = 1) 
   a <- as.vector(x)
   b <- as.vector(y)
   a.SE <- as.vector(a.SE)
   b.SE <- as.vector(b.SE)
   while(nrow(x) <= 2 || sqrt(var(abs(x[nrow(x),] - x[nrow(x)-1,]))) > 0.0001 || sqrt(var(abs(y[nrow(y),] - y[nrow(y)-1,]))) > 0.0001) {
      hiq <- matrix(0, ncol = length(Aq), nrow = nrow(u))
      rjq <- Njq <- matrix(0, ncol = length(Aq), nrow = ncol(u))
      Njq <- c(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0)
      for(i in 1:nrow(u)) {
         hiq[i,] <- mlfunc(a, b, Aq, u[i,], mu, sigma) / sum(mlfunc(a, b, Aq, u[i,], mu, sigma))
         Njq <- Njq + hiq[i,]
      }
      for(i in 1:ncol(u)) {
         for(j in 1:length(Aq)) {
            rjq[i,j] <- sum(u[,i] * hiq[,j])
         }
      }
      for(i in 1:ncol(u)) {
         lambda <- matrix(c(a[i], b[i]), ncol=1, nrow=2) + solve(Ifunc(a[i], b[i], Aq, Njq)) %*% roundm(a[i], b[i], Aq, rjq[i,], Njq)
         while((ncol(lambda) <= 2 || abs(rev(lambda[1,])[2] - rev(lambda[1,])[1]) > 0.00001 || abs(rev(lambda[2,])[2] - rev(lambda[2,])[1]) > 0.00001) && (lambda[1,ncol(lambda)] > 0 && lambda[1,ncol(lambda)] < 2 && abs(lambda[2,ncol(lambda)]) < 4) && ncol(lambda) <= 100) {
            lambda <- cbind(lambda, lambda[,ncol(lambda)] + solve(Ifunc(lambda[1,ncol(lambda)], lambda[2,ncol(lambda)], Aq, Njq)) %*% roundm(lambda[1,ncol(lambda)], lambda[2,ncol(lambda)], Aq, rjq[i,], Njq))
         }
         if(((lambda[1,ncol(lambda)] > 0 && lambda[1,ncol(lambda)] < 2) || abs(lambda[2,ncol(lambda)]) < 4) == T) {
            a[i] <- lambda[1,ncol(lambda)]
            b[i] <- lambda[2,ncol(lambda)]
         }
      }
      x <- rbind(x, a)
      y <- rbind(y, b)
   }
   for(i in 1:ncol(u)) {
      a.SE[i] <- sqrt(diag(solve(Ifunc(x[nrow(x),i], y[nrow(y),i] , Aq, Njq))))[1]
      b.SE[i] <- sqrt(diag(solve(Ifunc(x[nrow(x),i], y[nrow(y),i] , Aq, Njq))))[2]
   }
   list(a = x[nrow(x),], a.SE = a.SE, b = y[nrow(y),], b.SE = b.SE)
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#
#  >>> input <<<
#           u: item response data ; matrix
#          mu: mean of popular distribution
#       sigma: standard deviation of popular distribution
#
#  >>> output <<<
#           a: discrimination parameter estimate
#        a.SE: standard error of discrimination parameter estimate
#           b: difficulty parameter estimate
#        b.SE: standard error of difficulty parameter estimate
#  
#
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#                                               02/25/04  by  Tanase
}

IRT.par.freq <- function(u, k, mu, sigma) {
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<
#>>>>                                              <<<<<
#>>>>   Estimate item parameter in IRT             <<<<<
#>>>>                                              <<<<<
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<
   Aq <- c(-3.2, -2.8, -2.4, -2.0, -1.6, -1.2, -0.8, -0.4, 0, 0.4, 0.8, 1.2, 1.6, 2.0, 2.4, 2.8, 3.2)
   x <- matrix(1, ncol = ncol(u), nrow = 1)
   y <- a.SE <- b.SE <- matrix(0, ncol = ncol(u), nrow = 1) 
   a <- as.vector(x)
   b <- as.vector(y)
   a.SE <- as.vector(a.SE)
   b.SE <- as.vector(b.SE)
   while(nrow(x) <= 2 || sqrt(var(abs(x[nrow(x),] - x[nrow(x)-1,]))) > 0.00001 || sqrt(var(abs(y[nrow(y),] - y[nrow(y)-1,]))) > 0.00001) {
      hiq <- matrix(0, ncol = length(Aq), nrow = nrow(u))
      rjq <- Njq <- matrix(0, ncol = length(Aq), nrow = ncol(u))
      Njq <- c(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0) 
      for(i in 1:nrow(u)) {
         hiq[i,] <- mlfunc(a, b, Aq, u[i,], mu, sigma) / sum(mlfunc(a, b, Aq, u[i,], mu, sigma))
         Njq <- Njq + k[i] * hiq[i,]
      }
      for(i in 1:ncol(u)) {
         for(j in 1:length(Aq)) {
            rjq[i,j] <- sum(u[,i] * k * hiq[,j])
         }
      }
      for(i in 1:ncol(u)) {
         lambda <- matrix(c(a[i], b[i]), ncol=1, nrow=2) + solve(Ifunc(a[i], b[i], Aq, Njq)) %*% roundm(a[i], b[i], Aq, rjq[i,], Njq)
         while(ncol(lambda) <= 2 || abs(rev(lambda[1,])[2] - rev(lambda[1,])[1]) > 0.00001 || abs(rev(lambda[2,])[2] - rev(lambda[2,])[1]) > 0.00001) {
            lambda <- cbind(lambda, lambda[,ncol(lambda)] + solve(Ifunc(lambda[1,ncol(lambda)], lambda[2,ncol(lambda)], Aq, Njq)) %*% roundm(lambda[1,ncol(lambda)], lambda[2,ncol(lambda)], Aq, rjq[i,], Njq))
         }
         a[i] <- lambda[1,ncol(lambda)]
         b[i] <- lambda[2,ncol(lambda)]
      }
      x <- rbind(x, a)
      y <- rbind(y, b)
   }  
   for(i in 1:ncol(u)) {
      a.SE[i] <- sqrt(diag(solve(Ifunc(x[nrow(x),i], y[nrow(y),i] , Aq, Njq))))[1]
      b.SE[i] <- sqrt(diag(solve(Ifunc(x[nrow(x),i], y[nrow(y),i] , Aq, Njq))))[2]
   }
   list(a = x[nrow(x),], a.SE = a.SE, b = y[nrow(y),], b.SE = b.SE)
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#
#  >>> input <<<
#           u: item response data ; matrix
#           k: freqquency vector
#          mu: mean of popular distribution
#       sigma: standard deviation of popular distribution
#
#  >>> output <<<
#           a: discrimination parameter estimate
#        a.SE: standard error of discrimination parameter estimate
#           b: difficulty parameter estimate
#        b.SE: standard error of difficulty parameter estimate
#  
#
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#                                               02/25/04  by  Tanase
}

IRT.eq <- function(a, b, A, B) {
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#>>>>                                                       <<<<<
#>>>>   Estimate equating parameters  in IRT                <<<<<
#>>>>                                                       <<<<<
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
   alpha <- 2 * sum(a * A) / sum(a ^ 2 - A ^ 2)
   if(alpha > 0) {
      sgn <- 1
   } else if(alpha == 0) {
      sgn <- 0
   } else sgn <- -1
   k <- alpha / (-1 + sgn * sqrt(1 + alpha ^ 2))
   l <- mean(b) - k * mean(B)
   a <- a / k  
   b <- b * k + l
   list(k = k, l = l, a = a, b = b)
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#
#  >>> input <<<
#           a: discrimination parameter vector
#           b: difficulty parameter vector
#           A: discrimination parameter vector
#           B: difficulty parameter vector
#
#  >>> output <<<
#           estimate equating parameter
#           k: slope parameter
#           l: intersepte parameter
#           a: equated discrimination parameter vector
#           b: equated difficulty parameter vector
#
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#                                               02/25/04  by  Tanase
}

IRT.curv <- function(a, b) {
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#>>>>                                                       <<<<<
#>>>>   Plot test information curve in IRT                  <<<<<
#>>>>                                                       <<<<<
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
      trait <- seq(-5, 5, length=100)
      jouhou <- seq(0, 0, length=100)
      for(j in 1:length(a)) {
         for(i in 1:100) {
            jouhou[i] <- jouhou[i] +  1.7 ^ 2 * sum(a[j] ^ 2 * charfunc(a[j], b[j], trait[i]) * (1 - charfunc(a[j], b[j], trait[i])))
         }
      }
      plot(trait, jouhou, type = "l")
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#
#  >>> input <<<
#           a: discrimination parameter vector
#           b: difficulty parameter vector
# 
#  >>> output <<<
#           a graph of test information curve     
#  
#
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#                                               02/25/04  by  Tanase
}

charfunc <- function(a, b, theta) {
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#>>>>                                                                 <<<<<
#>>>>   Item Characteristic Function,  two-parameter logstic model    <<<<<
#>>>>    using in IRT.curv, IRT.trait, f.func, g.func, h.func,        <<<<<
#>>>>    mlfunc, roundlambda, roundm, Ifunc                           <<<<<
#>>>>                                                                 <<<<<
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
   1 / (1 + exp(-1.7 * a * (theta - b)))
#<<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>><<<<<>>>>>
#                                               02/25/04  by  Tanase
}

f.func <- function(a, b, theta, u) {
# Using from IRT.bayes
#                                02/25/04 by Tanase
   L <- 1
   for(i in 1:length(a)){
      L <- L * (charfunc(a[i], b[i], theta) ^ u[i] * (1 - charfunc(a[i], b[i], theta)) ^ (1 - u[i]))
   }
   L
}


g.func <- function(a, b, theta, u) {
# Using from IRT.trait
#                                02/25/04 by Tanase
   L <- 0
   for(i in 1:length(a)) {
      L <- L + 1.7 * a[i] * (u[i] - charfunc(a[i], b[i], theta))
   }
   L
}

h.func <- function(a, b, theta, u) {
# Using from IRT.trait
#                                02/25/04 by Tanase
   L <- 0
   for(i in 1:length(a)){
      L <- L + (-1.7 ^ 2 * a[i] ^ 2 * charfunc(a[i], b[i], theta) * (1 - charfunc(a[i], b[i], theta)))
   }
   L
}

mlfunc <- function(a, b, Aq, u, mu, sigma) {
# Using from IRT.dist, IRT.dist.freq, IRT.par, IRT.par.freq
#                                    02/25/04 by Tanase
   f <- 1
   for(i in 1:length(u)) {
      f <- f * charfunc(a[i], b[i], Aq) ^ u[i] * (1 - charfunc(a[i], b[i], Aq)) ^ (1 - u[i]) 
   }
   f <- f * (dnorm(Aq, mu, sigma) / sum(dnorm(Aq, mu, sigma)))
   f
}

roundlambda <- function(a, b, theta) {
# Using from IRT.par, IRT.par.freq, roundm, Ifunc
#                                02/25/04 by Tanase
   lambda <- matrix(0, ncol=length(theta), nrow=2)
   lambda[1,] <- 1.7 * (theta - b) * charfunc(a, b, theta) * (1 - charfunc(a, b, theta))
   lambda[2,] <- -1.7 * a * charfunc(a, b, theta) * (1 - charfunc(a, b, theta))
   lambda
}

roundm <- function(a, b, theta, rjq, Njq) {
# Using from IRT.par, IRT.par.freq
#                                02/25/04 by Tanase
   lambda <- matrix(0, ncol=1, nrow=2)
   lambda[1,1] <- sum((rjq - Njq * charfunc(a, b, theta)) / (charfunc(a, b, theta) * (1 - charfunc(a, b, theta))) * roundlambda(a, b, theta)[1,])
   lambda[2,1] <- sum((rjq - Njq * charfunc(a, b, theta)) / (charfunc(a, b, theta) * (1 - charfunc(a, b, theta))) * roundlambda(a, b, theta)[2,])
   lambda
}

Ifunc <- function(a, b, theta, Njq) {
# Using from IRT.par, IRT.par.freq
#                                02/25/04 by Tanase
   lambda <- matrix(0, ncol = 2, nrow = 2)
   for(i in 1:length(theta)) { 
      lambda <- lambda + Njq[i] / (charfunc(a, b, theta[i]) * (1 - charfunc(a, b, theta[i]))) * matrix(roundlambda(a, b, theta)[,i], ncol = 1, nrow = 2) %*% t(matrix(roundlambda(a, b, theta)[,i], ncol = 1, nrow = 2))
   }
   lambda
}
